import os
from pynwb import load_namespaces

# Set path of the namespace.yaml file to the expected install location
ndx_ecog_specpath = os.path.join(
    os.path.dirname(__file__), 
    'spec', 
    'ndx-ecog.namespace.yaml'
)

# If the extension has not been installed yet but we are running directly from the git repo
if not os.path.exists(ndx_ecog_specpath):
    ndx_ecog_specpath = os.path.abspath(os.path.join(
        os.path.dirname(__file__),
        '..', '..', '..', 
        'spec',
        'ndx-ecog.namespace.yaml'
    ))

# Load the namespace 
load_namespaces(ndx_ecog_specpath)

from .ecog import ECoGSubject, CorticalSurfaces, Surface




from collections import Iterable

import numpy as np
from hdmf.utils import popargs, get_docval, docval
from pynwb import register_class
from pynwb.core import NWBContainer, MultiContainerInterface
from pynwb.file import Subject
from pynwb.base import Images


@register_class('Surface', 'ndx-ecog')
class Surface(NWBContainer):
    __nwbfields__ = ('faces', 'vertices')

    @docval({'name': 'name', 'type': str, 'doc': 'name'},
            {'name': 'vertices', 'type': Iterable, 'shape': (None, 3),
             'doc': 'vertices for surface, points in 3D space'},
            {'name': 'faces', 'type': Iterable, 'shape': (None, 3),
             'doc': 'faces for surface, indexes vertices'})
    def __init__(self, **kwargs):
        name, vertices, faces = popargs('name', 'vertices', 'faces', kwargs)
        super(Surface, self).__init__(name)
        if np.max(faces) >= len(vertices):
            raise ValueError('index of faces exceeds number vertices for {}. '
                             'Faces should be 0-indexed, not 1-indexed'.
                             format(name))
        if np.min(faces) < 0:
            raise ValueError('faces hold indices of vertices and should be non-negative')
        self.faces = faces
        self.vertices = vertices


@register_class('CorticalSurfaces', 'ndx-ecog')
class CorticalSurfaces(MultiContainerInterface):
    """
    Spike data for spike events detected in raw data
    stored in this NWBFile, or events detect at acquisition
    """

    __clsconf__ = {
        'attr': 'surfaces',
        'type': Surface,
        'add': 'add_surface',
        'get': 'get_surface',
        'create': 'create_surface'
    }

    __help = "triverts for cortical surfaces"


@register_class('ECoGSubject', 'ndx-ecog')
class ECoGSubject(Subject):

    __nwbfields__ = ({'name': 'cortical_surfaces', 'child': True},
                     {'name': 'images', 'child': True})

    @docval(*get_docval(Subject.__init__) + (
        {
            'name': 'cortical_surfaces',
            'doc': 'extension of Subject that allows adding cortical surface data',
            'type': CorticalSurfaces,
            'default': None,
        },
        {
            'name': 'images',
            'doc': "Images of subject's brain",
            'type': Images,
            'default': None
        })
            )
    def __init__(self, **kwargs):
        cortical_surfaces, images = popargs('cortical_surfaces', 'images', kwargs)
        super(ECoGSubject, self).__init__(**kwargs)
        self.cortical_surfaces = cortical_surfaces
        self.images = images



# -*- coding: utf-8 -*-
# Configuration file for generating sources for the format documentation from the YAML specification files

import os

# -- Input options for the specification files to be used -----------------------

# Directory where the YAML files for the namespace to be documented are located
spec_input_spec_dir = '../spec'

# Name of the YAML file with the specification of the Namespace to be documented
spec_input_namespace_filename = 'ndx-ecog.namespace.yaml'

# Name of the default namespace in the file
spec_input_default_namespace = 'ndx-ecog'


# -- Options for customizing the locations of output files

# Directory where the autogenerated files should be stored
spec_output_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)), "_format_auto_docs")

# Name of the master rst file that includes all the autogenerated docs
spec_output_master_filename = 'format_spec_main.inc'

# Name of the file where the main documentation goes
spec_output_doc_filename = 'format_spec_doc.inc'

# Name of the file where the sources of the format spec go. NOTE: This file is only generated if
# spec_generate_src_file is enabled
spec_output_src_filename = 'format_spec_sources.inc'

# Name of the file containing the type hierarchy. (Included in spec_output_doc_filename)
spec_output_doc_type_hierarchy_filename = 'format_spec_type_hierarchy.inc'

# Clean up the output directory before we build if the git hash is out of date
spec_clean_output_dir_if_old_git_hash = True

# Do not rebuild the format sources if we have previously build the sources and the git hash matches
spec_skip_doc_autogen_if_current_git_hash = False


# -- Options for the generation of the documentation from source ----------------

# Should the YAML sources be included for the different modules
spec_show_yaml_src = True

# Show figure of the hierarchy of objects defined by the spec
spec_show_hierarchy_plots = True

# Should the sources of the neurodata_types (YAML) be rendered in a separate section (True) or
# in the same location as the base documentation
spec_generate_src_file = True

# Should separate .inc reStructuredText files be generated for each neurodata_type (True)
# or should all text be added to the main file
spec_file_per_type = True

# Should top-level subgroups be listed in a separate table or as part of the main dataset and attributes table
spec_show_subgroups_in_seperate_table = True

# Abbreviate the documentation of the main object for which a table is rendered in the table.
# This is commonly set to True as doc of the main object is alrready rendered as the main intro for the
# section describing the object
spec_appreviate_main_object_doc_in_tables = True

# Show a title for the tables
spec_show_title_for_tables = True

# Char to be used as prefix to indicate the depth of an object in the specification hierarchy
spec_table_depth_char = '.'  # '→' '.'

# Add a LaTeX clearpage after each main section describing a neurodata_type. This helps in LaTeX to keep the ordering
# of figures, tables, and code blocks consistent in particular when the hierarchy_plots are included
spec_add_latex_clearpage_after_ndt_sections = True

# Resolve includes to always show the full list of objects that are part of a type (True)
# or to show only the parts that are actually new to a current type while only linking to base types
spec_resolve_type_inc = False

# Default type map to be used. This is the type map where dependent namespaces are stored. In the case of
# NWB this is spec_default_type_map = pynwb.get_type_map()
import pynwb  # noqa: E402
spec_default_type_map = pynwb.get_type_map()

# Default specification classes for groups datasets and namespaces. In the case of NWB these are the NWB-specfic
# spec classes. In the general cases these are the spec classes from HDMF
spec_group_spec_cls = pynwb.spec.NWBGroupSpec
spec_dataset_spec_cls = pynwb.spec.NWBDatasetSpec
spec_namespace_spec_cls = pynwb.spec.NWBNamespace



from datetime import datetime
import os

import numpy as np
import unittest2 as unittest
from pynwb import NWBFile, NWBHDF5IO

from ndx_ecog import ECoGSubject, CorticalSurfaces


class ECoGSubjectTest(unittest.TestCase):

    def setUp(self):
        self.vertices = np.random.randn(20, 3)
        self.faces = np.random.randint(0, 20, (10, 3)).astype('uint')
        self.nwbfile = NWBFile('description', 'id', datetime.now().astimezone())

    def test_init_ecog_subject(self):

        cortical_surfaces = CorticalSurfaces()
        cortical_surfaces.create_surface('test', vertices=self.vertices, faces=self.faces)
        self.nwbfile.subject = ECoGSubject(subject_id='id', cortical_surfaces=cortical_surfaces)

        np.testing.assert_allclose(self.nwbfile.subject.cortical_surfaces.surfaces['test'].vertices, self.vertices)
        np.testing.assert_allclose(self.nwbfile.subject.cortical_surfaces.surfaces['test'].faces, self.faces)

    def test_add_cs_to_ecog_subject(self):

        cortical_surfaces = CorticalSurfaces()
        cortical_surfaces.create_surface('test', vertices=self.vertices, faces=self.faces)
        self.nwbfile.subject = ECoGSubject()
        self.nwbfile.subject.cortical_surfaces = cortical_surfaces

    def test_io(self):

        cortical_surfaces = CorticalSurfaces()
        cortical_surfaces.create_surface('test', vertices=self.vertices, faces=self.faces)
        self.nwbfile.subject = ECoGSubject(subject_id='id', cortical_surfaces=cortical_surfaces)

        with NWBHDF5IO('test.nwb', 'w') as io:
            io.write(self.nwbfile)

        with NWBHDF5IO('test.nwb', 'r') as io:
            nwbfile = io.read()
            np.testing.assert_allclose(
                self.nwbfile.subject.cortical_surfaces.surfaces['test'].vertices,
                nwbfile.subject.cortical_surfaces.surfaces['test'].vertices)
            np.testing.assert_allclose(
                self.nwbfile.subject.cortical_surfaces.surfaces['test'].faces,
                nwbfile.subject.cortical_surfaces.surfaces['test'].faces)

        os.remove('test.nwb')



# -*- coding: utf-8 -*-

import os

from setuptools import setup, find_packages
from shutil import copy2

with open("README.md", "r") as fh:
    long_description = fh.read()

setup_args = {
    'name': 'ndx-ecog',
    'version': '0.1.0',
    'description': 'An NWB extension for storing the cortical surface of subjects in ECoG experiments',
    'author': 'Ben Dichter',
    'author_email': 'ben.dichter@gmail.com',
    'url': '',
    'long_description': long_description,
    'long_description_content_type': "text/markdown",
    'license': 'BSD 3-Clause',
    'install_requires': [
        'pynwb'
    ],
    'packages': find_packages('src/pynwb'),
    'package_dir': {'': 'src/pynwb'},
    'package_data': {'ndx_ecog': [
        'spec/ndx-ecog.namespace.yaml',
        'spec/ndx-ecog.extensions.yaml',
    ]},
    'classifiers': [
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
    ],
    'zip_safe': False
}


def _copy_spec_files(project_dir):
    ns_path = os.path.join(project_dir, 'spec', 'ndx-ecog.namespace.yaml')
    ext_path = os.path.join(project_dir, 'spec', 'ndx-ecog.extensions.yaml')

    dst_dir = os.path.join(project_dir, 'src', 'pynwb', 'ndx_ecog', 'spec')
    if not os.path.exists(dst_dir):
        os.mkdir(dst_dir)

    copy2(ns_path, dst_dir)
    copy2(ext_path, dst_dir)


if __name__ == '__main__':
    _copy_spec_files(os.path.dirname(__file__))
    setup(**setup_args)
